package jamawrapper;

import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import java.io.IOException;
import java.util.Random;

import Jama.Matrix;
import Jama.QRDecomposition;
import Jama.CholeskyDecomposition;
import Jama.EigenvalueDecomposition;
import Jama.LUDecomposition;
import Jama.SingularValueDecomposition;



@BA.Version(1.0f)
@BA.Author("Johan Schoeman")
@BA.ShortName("JamaMatrix")
public class jamaWrapper
{

  private Matrix jmx;
  private Matrix D;
  private Matrix V;
  private EigenvalueDecomposition Eig;
  
  jamaWrapper(Matrix m){
    jmx = m;
  }
  
  public jamaWrapper() {
  }


   /** Construct an m-by-n matrix of zeros. 
   *
   *m    Number of rows.
   *n    Number of colums.
   *
   */
   public void Initialize1 (int m, int n) {
      jmx = new Matrix(m, n);
   }


   /** Construct an m-by-n constant matrix.
   *
   *m    Number of rows.
   *n    Number of colums.
   *s    Fill the matrix with this scalar value.
   *
   */
   public void Initialize2 (int m, int n, double s) {
    jmx = new Matrix(m, n, s);
   }


   /** Construct a matrix from a 2-D array.
   *
   *A    Two-dimensional array of doubles.
   *
   */
   public void Initialize3 (double[][] A) {
    jmx = new Matrix(A);
   }


   /** Construct a matrix quickly without checking arguments.
   *
   *A    Two-dimensional array of doubles.
   *m    Number of rows.
   *n    Number of colums.
   *
   */
   public void Initialize4 (double[][] A, int m, int n) {
    jmx = new Matrix(A, m, n);
   }


   /** Construct a matrix from a one-dimensional packed array
   *
   *vals One-dimensional array of doubles, packed by columns (ala Fortran).
   *m    Number of rows.
   *
   */
   public void Initialize5 (double vals[], int m) {
    jmx = new Matrix(vals, m);
   }

   /** Make a deep copy of a matrix
   */
   public jamaWrapper copy(){
     return new jamaWrapper((Matrix)jmx.copy());
   }


   /** Construct a matrix from a copy of a 2-D array.
   *
   *A    Two-dimensional array of doubles.
   *
   */
   public jamaWrapper constructWithCopy(double[][] A) {
      return new jamaWrapper((Matrix)jmx.constructWithCopy(A));
   }


   /** Copy the internal two-dimensional array.
   *
   *return     Two-dimensional array copy of matrix elements.
   */
   public double[][] getArrayCopy () {
      return jmx.getArrayCopy();
   }


   /** Make a one-dimensional column packed copy of the internal array.
   *
   *return     Matrix elements packed in a one-dimensional array by columns.
   *The returned one-dimensional array is of type Double
   */
   public double[] getColumnPackedCopy() {
      return jmx.getColumnPackedCopy();
   }


   /** Make a one-dimensional row packed copy of the internal array.
   *
   *return     Matrix elements packed in a one-dimensional array by rows.
   *The returned one-dimensional array is of type Double
   */
   public double[] getRowPackedCopy() {
      return jmx.getRowPackedCopy();
   }


   /** Get row dimension.
   *return     m (an int), the number of rows.
   */
   public int getRowDimension() {
      return jmx.getRowDimension();
   }


   /** Get column dimension.
   *
   *return     n (an int), the number of columns.
   */
   public int getColumnDimension() {
      return jmx.getColumnDimension();
   }


   /** Return a single element (type Double)
   *
   *i    Row index.
   *j    Column index.
   *return     A(i,j)
   *
   */
   public double ReturnSingleElement (int i, int j) {
      return jmx.get(i, j);
   }


   /** Get a submatrix.
   *
   *i0   Initial row index
   *i1   Final row index
   *j0   Initial column index
   *j1   Final column index
   *return     A(i0:i1,j0:j1)
   *
   */
   public jamaWrapper GetSubMatrix1 (int i0, int i1, int j0, int j1) {
      return new jamaWrapper((Matrix)jmx.getMatrix(i0, i1, j0, j1));
   }


   /** Get a submatrix.
   *
   *r    Array of row indices.
   *c    Array of column indices.
   *return     A(r(:),c(:))
   *
   */
   public jamaWrapper GetSubMatrix2 (int[] r, int[] c) {
      return new jamaWrapper((Matrix)jmx.getMatrix(r, c));
   }


   /** Get a submatrix.
   *
   *i0   Initial row index
   *i1   Final row index
   *c    Array of column indices.
   *return     A(i0:i1,c(:))
   *
   */
   public jamaWrapper GetSubMatrix3 (int i0, int i1, int[] c) {
      return new jamaWrapper((Matrix)jmx.getMatrix(i0, i1, c));
   }


   /** Get a submatrix.
   *
   *r    Array of row indices.
   *j0   Initial column index
   *j1   Final column index
   *return     A(r(:),j0:j1)
   *
   */
   public jamaWrapper GetSubMatrix4 (int[] r, int j0, int j1) {
      return new jamaWrapper((Matrix)jmx.getMatrix(r, j0, j1));
   }


   /** Set a single element.
   *i    Row index.
   *j    Column index.
   *s    A(i,j).
   *
   */
   public void SetSingleElement(int i, int j, double s) {
      jmx.set(i, j, s);
   }


   /** Set a submatrix.
   *
   *i0   Initial row index
   *i1   Final row index
   *j0   Initial column index
   *j1   Final column index
   *X    A(i0:i1,j0:j1)
   *
   */
   public void SetSubMatrix1 (int i0, int i1, int j0, int j1, jamaWrapper X) {
      jmx.setMatrix (i0, i1, j0, j1, X.jmx);
   }


   /** Set a submatrix.
   *
   *r    Array of row indices.
   *c    Array of column indices.
   *X    A(r(:),c(:))
   *
   */
   public void SetSubMatrix2 (int[] r, int[] c, jamaWrapper X) {
      jmx.setMatrix (r, c, X.jmx);
   }


   /** Set a submatrix.
   *
   *r    Array of row indices.
   *j0   Initial column index
   *j1   Final column index
   *X    A(r(:),j0:j1)
   *
   */
   public void SetSubMatrix3 (int[] r, int j0, int j1, jamaWrapper X) {
      jmx.setMatrix (r, j0, j1, X.jmx);
   }


   /** Set a submatrix.
   *
   *i0   Initial row index
   *i1   Final row index
   *c    Array of column indices.
   *X    A(i0:i1,c(:))
   *
   */
   public void SetSubMatrix4 (int i0, int i1, int[] c, jamaWrapper X) {
      jmx.setMatrix (i0, i1, c, X.jmx);
   }


   /** Matrix transpose.
   *
   *return    A'
   */
   public jamaWrapper Transpose () {
      return new jamaWrapper((Matrix)jmx.transpose());
   }


   /** One norm
   *return    maximum column sum.
   */
   public double Norm1() {
      return jmx.norm1();
   }


   /** Two norm
   *return    maximum singular value.
   */
   public double Norm2 () {
      return jmx.norm2();
   }


   /** Infinity norm
   *return    maximum row sum.
   */
   public double NormInf () {
      return jmx.normInf();
   }


   /** Frobenius norm
   *return    sqrt of sum of squares of all elements.
   */

   public double NormF () {
      return jmx.normF();
   }



   /**  Unary minus
   *return    -A
   */
   public jamaWrapper uminus () {
      return new jamaWrapper((Matrix)jmx.uminus());
   }


   /** C = A + B
   *
   *B    another matrix
   *return     A + B
   */
   public jamaWrapper plus (jamaWrapper B) {
      return new jamaWrapper((Matrix)jmx.plus(B.jmx));
   }


   /** A = A + B
   *
   *B    another matrix
   *return     A + B
   */
   public jamaWrapper plusEquals (jamaWrapper B) {
      return new jamaWrapper((Matrix)jmx.plusEquals(B.jmx));
   }


   /** C = A - B
   *
   *B    another matrix
   *return     A - B
   */
   public jamaWrapper minus (jamaWrapper B) {
      return new jamaWrapper((Matrix)jmx.minus(B.jmx));
   }


   /** A = A - B
   *
   *B          another matrix
   *return     A - B
   */
   public jamaWrapper minusEquals (jamaWrapper B) {
      return new jamaWrapper((Matrix)jmx.minusEquals(B.jmx));
   }


   /** Element-by-element multiplication, C = A.*B
   *
   *B    another matrix
   *return     A.*B
   */
   public jamaWrapper arrayTimes (jamaWrapper B) {
      return new jamaWrapper((Matrix)jmx.arrayTimes(B.jmx));
   }


   /** Element-by-element multiplication in place, A = A.*B
   *
   *B    another matrix
   *return     A.*B
   */
   public jamaWrapper arrayTimesEquals (jamaWrapper B) {
      return new jamaWrapper((Matrix)jmx.arrayTimesEquals(B.jmx));
   }


   /** Element-by-element right division, C = A./B
   *
   *B    another matrix
   *return     A./B
   */
   public jamaWrapper arrayRightDivide (jamaWrapper B) {
      return new jamaWrapper((Matrix)jmx.arrayRightDivide(B.jmx));
   }


   /** Element-by-element right division in place, A = A./B
   *
   *B    another matrix
   *return     A./B
   */
   public jamaWrapper arrayRightDivideEquals (jamaWrapper B) {
      return new jamaWrapper((Matrix)jmx.arrayRightDivideEquals(B.jmx));
   }


   /** Element-by-element left division, C = A.\B
   *
   *B    another matrix
   *return     A.\B
   */
   public jamaWrapper arrayLeftDivide (jamaWrapper B) {
      return new jamaWrapper((Matrix)jmx.arrayLeftDivide(B.jmx));
   }


   /** Element-by-element left division in place, A = A.\B
   *
   *B    another matrix
   *return     A.\B
   */
   public jamaWrapper arrayLeftDivideEquals (jamaWrapper B) {
      return new jamaWrapper((Matrix)jmx.arrayLeftDivideEquals(B.jmx));
   }


   /** Multiply a matrix by a scalar, C = s*A
   *
   *s    scalar
   *return     s*A
   */
   public jamaWrapper timesByScalar (double s) {
      return new jamaWrapper((Matrix)jmx.times(s));
   }


   /** Multiply a matrix by a scalar in place, A = s*A
   *
   *s    scalar
   *return     replace A by s*A
   */
   public jamaWrapper timesEquals (double s) {
      return new jamaWrapper((Matrix)jmx.timesEquals(s));
   }


   /** Linear algebraic matrix multiplication, A * B
   *
   *B    another matrix
   *return     Matrix product, A * B
   *
   */
   public jamaWrapper timesByMatrix (jamaWrapper B) {
      return new jamaWrapper((Matrix)jmx.times(B.jmx));
   }


   /** Solve A*X = B
   *
   *B    right hand side
   *return     solution if A is square, least squares solution otherwise
   */
   public jamaWrapper solve (jamaWrapper B) {
      return new jamaWrapper((Matrix)jmx.solve(B.jmx));
   }


   /** Solve X*A = B, which is also A'*X' = B'
   *
   *B    right hand side
   *return     solution if A is square, least squares solution otherwise.
   */
   public jamaWrapper solveTranspose (jamaWrapper B) {
      return new jamaWrapper((Matrix)jmx.solveTranspose(B.jmx));
   }


   /** Matrix inverse or pseudoinverse
   *return     inverse(A) if A is square, pseudoinverse otherwise.
   */
   public jamaWrapper inverse () {
      return new jamaWrapper((Matrix)jmx.inverse());
   }


   /** Matrix determinant
   *return     determinant
   */
   public double det () {
      return jmx.det();
   }


   /** Matrix rank
   *return     effective numerical rank, obtained from SVD.
   */
   public int rank () {
      return jmx.rank();
   }


   /** Matrix condition (2 norm)
   *return     ratio of largest to smallest singular value.
   */
   public double condition () {
      return jmx.cond();
   }


   /** Matrix trace.
   *return     sum of the diagonal elements.
   */
   public double trace () {
      return jmx.trace();
   }


   /** Generate matrix with random elements
   *
   *m    Number of rows.
   *n    Number of colums.
   *return     An m-by-n matrix with uniformly distributed random elements.
   */
   public jamaWrapper random (int m, int n) {
      return new jamaWrapper((Matrix)jmx.random(m, n));
   }


   /** Generate identity matrix
   *
   *m    Number of rows.
   *n    Number of colums.
   *return     An m-by-n matrix with ones on the diagonal and zeros elsewhere.
   */
   public jamaWrapper identity (int m, int n) {
      return new jamaWrapper((Matrix)jmx.identity(m, n));
   }


   /** Return the elements of the matrix as a two-dimensional array of doubles.
   *   
   */   
   public double[][] GetMatrixEntries() {
      return jmx.getArray();
   }




   /** Generate the eigenvalue decomposition
   *   of the initialized array
   */   
//   public void eigenValueDecomposition () {
//      Eig = jmx.eig();
//      D = Eig.getD();
//      V = Eig.getV();
//   }


   /** Return the block diagonal eigenvalue matrix
   *return     D
   */
//   public jamaWrapper getD () {
//      return new jamaWrapper((Matrix)Eig.getD());
//   }


   /** Return the eigenvector matrix
   *return     V
   */
//   public jamaWrapper getV () {
//      return new jamaWrapper((Matrix)Eig.getV());
//   }




}